package com.engine.attendance.attendanceanalysis.service.impl;

import com.engine.attendance.attendanceanalysis.cmd.GetClockInTimeCmd;
import com.engine.attendance.attendanceanalysis.service.UtilService;
import com.engine.attendance.attendanceplan.service.AttendancePlanService;
import com.engine.attendance.attendanceplan.service.impl.AttendancePlanServiceImpl;
import com.engine.attendance.component.persongroup.service.SchedulingResultsService;
import com.engine.attendance.component.persongroup.service.impl.SchedulingResultsServiceImpl;
import com.engine.attendance.enums.AttendanceItemTypeEnum;
import com.engine.attendance.enums.ClassSegmentTypeEnum;
import com.engine.attendance.enums.WorkForTimeEnum;
import com.engine.common.cmd.GetDateCmd;
import com.engine.common.util.DateUtil;
import com.engine.common.util.DbTools;
import com.engine.common.util.ServiceUtil;
import com.engine.core.impl.Service;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import lombok.extern.slf4j.Slf4j;
import weaver.general.Util;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Slf4j
public class UtilServiceImpl extends Service implements UtilService {
    private SchedulingResultsService schedulingResultsService = ServiceUtil.getService(SchedulingResultsServiceImpl.class);
    private AttendancePlanService basicsetService = ServiceUtil.getService(AttendancePlanServiceImpl.class);


    @Override
    public Map<String, List<Map<String,Object>>> getSchedulingInFormation(Map<String, Object> params) {

        /** 班次信息*/
        Map<String,Object> schedulingResultsMap = schedulingResultsService.queryDataTable(params);
        List<Map<String,Object>> schedulingResultsList = (List<Map<String,Object>>)schedulingResultsMap.get("data");
        schedulingResultsList = schedulingResultsList.stream().filter(e -> !"1".equals(e.get("sfxx"))).collect(Collectors.toList());
        Map<String,String> schedulingMap = schedulingResultsList.stream().collect(Collectors.toMap(e-> Util.null2String(e.get("bcxx")), e->Util.null2String(e.get("bcrq"))));
        String sql = "select a.id bcxx,a.edsc,a.bcsdxx,a.sfdx,a.sfkt,b.bdlx,b.zddxfz,b.dxhs,b.kssj dtkssj,b.jssj dtjssj,b.ksdk,b.jsdk,b.tqdkfzs,b.thdkfzs from uf_jcl_kq_bcxx a left join uf_jcl_kq_bcxx_dt1 b on a.id=b.mainid where a.id in (";
        String bcxxIds = "";
        for (Map.Entry<String,String> entry :schedulingMap.entrySet()){
            bcxxIds +=entry.getKey() +",";
        }
        bcxxIds = bcxxIds.substring(0,bcxxIds.length()-1);
        sql = sql +bcxxIds+") order by b.kssj";
        List<Map<String,Object>> dataList = DbTools.getSqlToList(sql);
        Map<String,List<Map<String,Object>>> dataMap = dataList.stream().collect(Collectors.groupingBy(e->Util.null2String(e.get("bcxx"))));
        Map<String,List<Map<String,Object>>> resultMap = Maps.newHashMap();
        dataMap.entrySet().forEach(e -> {
            resultMap.put(schedulingMap.get(e.getKey()),e.getValue());
        });

        /** 加班计划*/
        sql = "select jbry,ksrq,kssj,jsrq,jssj,jbsc from uf_jcl_kq_jbjh_dt1 where jbry=? and ksrq>=? and ksrq<=? and (jbcx=0 or jbcx is null)";
        Map<String,List<Map<String,Object>>> overtimePlanMap = DbTools.getSqlToList(sql,params.get("pbdx"),params.get("startDate"),params.get("endDate")).stream().collect(Collectors.groupingBy(e -> Util.null2String(e.get("ksrq"))));

        //考勤项目
        Map<String,Object> paramMap = Maps.newHashMap();
        paramMap.put("startDate","");
        paramMap.put("endDate","");
        paramMap.put("resourceId",params.get("pbdx"));
        paramMap.put("modeId","204");
        Map<String,Object> attendanceItemMap = basicsetService.getAttendanceItemsByPerson(paramMap);
        List<Map<String,Object>> attendanceItems = (List<Map<String,Object>>)attendanceItemMap.get("data");
        //日期集合
        Map<String,Object> dateParam = Maps.newHashMap();
        dateParam.put("nd",Util.null2String(params.get("startDate")).split("-")[0]);
        Map<String,Object> result = commandExecutor.execute(new GetDateCmd(dateParam));
        List<Map<String,Object>> List = (List<Map<String,Object>>)result.get("data");
        Map<String,String> dateMap = List.stream().collect(Collectors.toMap(e->Util.null2String(e.get("rq")),e->Util.null2String(e.get("rqlx"))));


        for (Map.Entry<String,List<Map<String,Object>>> e: overtimePlanMap.entrySet()){
            if (resultMap.get(e.getKey()) == null){
                resultMap.put(e.getKey(),Lists.newArrayList());
            }
            List<Map<String,Object>> resultList = resultMap.get(e.getKey());
            Map<String,Object> overtimePlan = e.getValue().get(0);


            Map<String,Object> map = Maps.newHashMap();
            if (!overtimePlan.get("ksrq").equals(overtimePlan.get("jsrq"))){
                map.put("sfkt","1");
            }else {
                map.put("sfkt","0");
            }
            //查找对应的考勤项目
            attendanceItems = attendanceItems.stream().filter(s->{
                //项目类型
                String xmlx = Util.null2String(s.get("xmlx"));
                //作用时段
                String zysd = Util.null2String(s.get("zysd"));
                //项目绑定的日期类型
                String bddrqlx = Util.null2String(s.get("bddrqlx"));
                //日期类型
                String dateType = dateMap.get(e.getKey());

                if (xmlx.equals(AttendanceItemTypeEnum.WORK_OVERTIME.getKey()) && (zysd.equals(WorkForTimeEnum.PLAN_WORK_OVERTIME.getKey()) || zysd.equals(WorkForTimeEnum.ALL_TIME.getKey())) && bddrqlx.contains(dateType)){
                    return true;
                }
                return false;
            }).collect(Collectors.toList());


            map.put("bcxx","0");
            map.put("bcsdxx","");
            map.put("edsc","8");
            map.put("bdlx", ClassSegmentTypeEnum.OVERTIME_PLAN.getKey());
            map.put("dtkssj",overtimePlan.get("kssj"));
            map.put("dtjssj",overtimePlan.get("jssj"));
            map.put("ksdk","0");
            map.put("jsdk","0");
            map.put("tqdkfzs","30");
            map.put("thdkfzs","30");
            map.put("sfdx","0");
            map.put("zddxfz","0");
            map.put("dxhs","0");
            if (attendanceItems.size() >0){
                map.put("ksdk",attendanceItems.get(0).get("ksjbbxydk"));
                map.put("jsdk",attendanceItems.get(0).get("jsjbbxydk"));
                map.put("tqdkfzs",attendanceItems.get(0).get("tqdkyxfzs"));
                map.put("thdkfzs",attendanceItems.get(0).get("thdkyxfzs"));
                map.put("jbwdhlfzs",attendanceItems.get(0).get("jbwdhlfzs"));
                map.put("jbzzhlfzs",attendanceItems.get(0).get("jbzzhlfzs"));
            }
            if (resultList.size() == 0){
                resultList.add(map);
            }
            for (int j=0;j<resultList.size();j++){
                String kssj1 = e.getKey() +" "+resultList.get(j).get("dtkssj");
                String kssj2 = e.getKey() +" "+overtimePlan.get("kssj");
                String kssj3 = e.getKey() +" "+resultList.get(j-1).get("dtkssj");
                if (DateUtil.getTime(kssj1).compareTo(DateUtil.getTime(kssj2)) >=0 &&  DateUtil.getTime(kssj3).compareTo(DateUtil.getTime(kssj2)) <=0){
                    map.put("bcxx",resultList.get(0).get("bcxx"));
                    map.put("bcsdxx",resultList.get(0).get("bcsdxx"));
                    map.put("edsc",resultList.get(0).get("edsc"));
                    resultList.add(j,map);
                }
            }

        }

        log.info("SchedulingResults : [{}]",resultMap);
        return resultMap;
    }


    @Override
    public List<Map<String, Object>> getClockInTime(Map<String, Object> params) {
        Map<String,Object> result = commandExecutor.execute(new GetClockInTimeCmd(params));
        List<Map<String, Object>> resultList = (List<Map<String, Object>>)result.get("resultList");
        return resultList;
    }


}
