package com.engine.attendance.attendanceanalysis.wrapper;

import com.engine.attendance.attendanceanalysis.cmd.UpdateAttendanceResultsCmd;
import com.engine.attendance.attendanceanalysis.service.BeLateService;
import com.engine.attendance.attendanceanalysis.service.ForgetClockInService;
import com.engine.attendance.attendanceanalysis.service.UtilService;
import com.engine.attendance.attendanceanalysis.service.impl.BeLateServiceImpl;
import com.engine.attendance.attendanceanalysis.service.impl.ForgetClockInServiceImpl;
import com.engine.attendance.attendanceanalysis.service.impl.UtilServiceImpl;
import com.engine.attendance.attendanceplan.service.AttendancePlanService;
import com.engine.attendance.attendanceplan.service.impl.AttendancePlanServiceImpl;
import com.engine.attendance.enums.CheckBoxEnum;
import com.engine.attendance.enums.ClassSegmentTypeEnum;
import com.engine.attendance.enums.PeriodOfTime;
import com.engine.common.util.DateUtil;
import com.engine.common.util.DbTools;
import com.engine.common.util.ServiceUtil;
import com.engine.common.util.Utils;
import com.engine.core.impl.Service;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.icbc.api.internal.apache.http.M;
import weaver.general.Util;

import java.time.ZoneOffset;
import java.util.Comparator;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 考勤分析主题逻辑
 */
public class AttendanceAnalysisWrapper extends Service {

    private AttendancePlanService basicsetService = ServiceUtil.getService(AttendancePlanServiceImpl.class);
    private BeLateService beLateService = ServiceUtil.getService(BeLateServiceImpl.class);
    private UtilService utilService = ServiceUtil.getService(UtilServiceImpl.class);
    private ForgetClockInService forgetClockInService = ServiceUtil.getService(ForgetClockInServiceImpl.class);

    public void attendanceAnalysis(String userId, List<Map<String,Object>> dataList){
        String beforeThreeDayDate = DateUtil.getCurrentDateMinusDay(3);
        String beforeTwoDayDate = DateUtil.getCurrentDateMinusDay(2);
        String beforeOneDayDate = DateUtil.getCurrentDateMinusDay(1);
        /** 打卡数据 */
        Map<String, List<Map<String,Object>>> collect = dataList.stream().collect(Collectors.groupingBy(e -> Util.null2String(e.get("signdate"))));

        /**获得人员考勤项目*/
        Map<String,Object> paramMap = Maps.newHashMap();
        paramMap.put("startDate","");
        paramMap.put("endDate","");
        paramMap.put("resourceId",userId);
        paramMap.put("modeId","204");
        Map<String,Object> dataMap = basicsetService.getAttendanceItemsByPerson(paramMap);
        List<Map<String,Object>> attendanceItems = (List<Map<String,Object>>)dataMap.get("data");

        /** 获取人员班次*/
        Map<String,Object> classesParamMap = Maps.newHashMap();
        classesParamMap.put("tableName","uf_pbjg");
        classesParamMap.put("startDate",beforeThreeDayDate);
        classesParamMap.put("endDate",beforeOneDayDate);
        classesParamMap.put("pblx","0");
        classesParamMap.put("pbdx",userId);
        classesParamMap.put("current","1");
        classesParamMap.put("pageSize",1);
        classesParamMap.put("recurrence",1);
        Map<String,List<Map<String,Object>>> schedulingResultsMap = utilService.getSchedulingInFormation(paramMap);

        /**获得人员加班计划*/
        String sql = "select jbry,ksrq,kssj,jsrq,jssj,jbsc from uf_jcl_kq_jbjh_dt1 where jbry=? and ksrq>=? and ksrq<=? and (jbcx=0 or jbcx is null)";
        Map<String,List<Map<String,Object>>> overtimePlanMap = DbTools.getSqlToList(sql,userId,beforeOneDayDate,beforeThreeDayDate).stream().collect(Collectors.groupingBy(e -> Util.null2String(e.get("ksrq"))));
        Map<String,Object> forgetClockInParam = Maps.newHashMap();
        forgetClockInParam.put("classInfo",schedulingResultsMap.get(beforeOneDayDate));
        forgetClockInParam.put("userId",schedulingResultsMap.get(beforeOneDayDate));


        /**计算获取前一天和第二天打卡数据*/
        Map<String,Object> getClockTimeParam = Maps.newHashMap();
        getClockTimeParam.put("date",beforeOneDayDate);
        getClockTimeParam.put("clockInTimeCollect",collect);
        getClockTimeParam.put("schedulingResultCollect",schedulingResultsMap);
        //前一天打卡数据
        List<Map<String, Object>> beforeOneDayClockInTimeList = utilService.getClockInTime(getClockTimeParam);

        getClockTimeParam.put("date",beforeTwoDayDate);
        //前第二天打卡数据
        List<Map<String, Object>> beforeTwoDayClockInTimeList = utilService.getClockInTime(getClockTimeParam);





        if (collect.size() == 0){
            //没有打卡数据，漏打卡
            forgetClockInParam.put("date",beforeOneDayDate);

            forgetClockInService.ForgetClockInAllDays(forgetClockInParam);
        }else if (collect.size() == 1){
            //有1天有打卡数据
            if (collect.get(beforeOneDayDate) != null){
                //前一天有打卡记录


            }else {
                //若为正常工作日，则记为漏打卡


            }
        }else if (collect.size() == 2){
            //有2天有打卡数据

            if (collect.get(beforeOneDayDate) == null){
                //若为正常工作日，前一天漏打卡


            }else if(collect.get(beforeOneDayDate) != null && collect.get(beforeTwoDayDate) == null){
                //前一天有打卡记录，第二天没有打卡记录


            }else if (collect.get(beforeTwoDayDate) != null && collect.get(beforeOneDayDate) != null){
                //前一天有打卡，第二天有打卡记录


            }


        }else if (collect.size() == 3){
            //连续3天打卡


        }
    }

    /**
     * 分析数据
     * @param userId 分析人员
     * @param analysisDate 分析日期
     * @param clockInTimeList 打卡数据
     * @param scheduleResult  班次
     * @param attendanceItems 考勤项目
     */
    public void analysis(String userId,String analysisDate,List<Map<String, Object>> clockInTimeList,List<Map<String, Object>> scheduleResult,List<Map<String,Object>> attendanceItems){
        if (scheduleResult == null || scheduleResult.size() ==0){
            //没有排班

            return;
        }
        List<Map<String, Object>> needClockInSchedule = scheduleResult.stream().filter(e -> "1".equals(e.get("ksdk")) || "1".equals(e.get("jsdk"))).collect(Collectors.toList());
        if(clockInTimeList == null || clockInTimeList.size() == 0){
            if (needClockInSchedule.size() > 0){
                //全天漏打卡
                return;
            }else {

            }
        }
        //请假记录
        String sql = "select a.qjry,a.jqlx,a.cxjqj,b.ksrq,b.kssj,b.jsrq,b.jssj,b.qjsc,b.qtj,b.btj from uf_jcl_kq_qjjl a left join uf_jcl_kq_qjjl_dt1 b on a.id = b.mainid where a.qjry=? and b.ksrq=? and (b.cxqj=0 or b.cxqj is null)";
        List<Map<String,Object>> askForLeaveList = DbTools.getSqlToList(sql,userId,analysisDate);

        //出差记录
        sql = "select a.ccr,a.cclx,a.ccsd,b.ksrq,b.jsrq,b.kssj,b.jssj,b.ccsc,b.qtcc,b.btcc,b.cxcc from uf_jcl_kq_ccjl a left join uf_jcl_kq_ccjl_dt1 b on a.id = b.mainid where a.ccr=? and a.ksrq=? and (b.cxcc=0 or b.cxcc is null)";
        List<Map<String,Object>> evectionList = DbTools.getSqlToList(sql,userId,analysisDate);

        Map<String,Object> updateAttendacneParams = Maps.newHashMap();
        updateAttendacneParams.put("userId",userId);
        updateAttendacneParams.put("date",analysisDate);
        updateAttendacneParams.put("classInfo",scheduleResult);

        sql = "select hsl,hsdw,lgsbxydk,tqlghlfzs,fgsbxydk,thfghlfzs,tybcndbjlhbjs,zdycbcndfgzsd,zdycrqqjndxxb from uf_jcl_kq_kqxm where id=?";
        for (Map<String,Object> askForLeaveData : askForLeaveList){
            //全天请假
            if (PeriodOfTime.ALLDAY.getKey().equals(askForLeaveData.get("cxjqj"))){
                updateAttendacneParams.put("item",askForLeaveData.get("jqlx"));
                updateAttendacneParams.put("itemduration",scheduleResult.get(0).get("edsc"));
                commandExecutor.execute(new UpdateAttendanceResultsCmd(updateAttendacneParams));
                return;
            }
            Map<String,Object> askForLeaveItem = DbTools.getSqlToMap(sql,askForLeaveData.get("jqlx"));
            Map<String,Object> askForLeaveMap = new HashMap(){{
                put("bcxx",scheduleResult.get(0).get("bcxx"));
                put("bcsdxx",scheduleResult.get(0).get("bcsdxx"));
                put("edsc",scheduleResult.get(0).get("edsc"));
                put("bdlx", ClassSegmentTypeEnum.ASK_FOR_LEAVE.getKey());
                put("dtkssj",askForLeaveData.get("kssj"));
                put("dtjssj",askForLeaveData.get("jssj"));
                put("ksdk",askForLeaveItem.get("fgsbxydk"));
                put("jsdk",askForLeaveItem.get("lgsbxydk"));
                put("tqdkfzs","60");
                put("thdkfzs","60");
                put("sfdx","1");
                put("zddxfz",askForLeaveItem.get("tqlghlfzs"));
                put("dxhs",askForLeaveItem.get("thfghlfzs"));
            }};

            scheduleResult.add(askForLeaveMap);
            if (CheckBoxEnum.CHECKED.getKey().equals(askForLeaveItem.get("lgsbxydk")) || CheckBoxEnum.CHECKED.getKey().equals(askForLeaveItem.get("fgsbxydk"))){
                needClockInSchedule.add(askForLeaveMap);
            }
        }
        for (Map<String,Object> evectionData :evectionList){
            //全天出差
            if (PeriodOfTime.ALLDAY.getKey().equals(evectionData.get("ccsd"))) {
                updateAttendacneParams.put("item", evectionData.get("cclx"));
                updateAttendacneParams.put("itemduration", scheduleResult.get(0).get("edsc"));
                commandExecutor.execute(new UpdateAttendanceResultsCmd(updateAttendacneParams));
                return;
            }
            Map<String,Object> evectionItem = DbTools.getSqlToMap(sql,evectionData.get("cclx"));
            Map<String,Object> evectionMap = new HashMap(){{
                put("bcxx",scheduleResult.get(0).get("bcxx"));
                put("bcsdxx",scheduleResult.get(0).get("bcsdxx"));
                put("edsc",scheduleResult.get(0).get("edsc"));
                put("bdlx", ClassSegmentTypeEnum.ASK_FOR_LEAVE.getKey());
                put("dtkssj",evectionData.get("kssj"));
                put("dtjssj",evectionData.get("jssj"));
                put("ksdk",evectionItem.get("fgsbxydk"));
                put("jsdk",evectionItem.get("lgsbxydk"));
                put("tqdkfzs","60");
                put("thdkfzs","60");
                put("sfdx","1");
                put("zddxfz",evectionItem.get("tqlghlfzs"));
                put("dxhs",evectionItem.get("thfghlfzs"));
            }};
            scheduleResult.add(evectionMap);
            if (CheckBoxEnum.CHECKED.getKey().equals(evectionItem.get("lgsbxydk")) || CheckBoxEnum.CHECKED.getKey().equals(evectionItem.get("fgsbxydk"))){
                needClockInSchedule.add(evectionMap);
            }
        }

        needClockInSchedule = needClockInSchedule.stream().sorted(Comparator.comparing(e->DateUtil.getTime(analysisDate+" "+e.get("dtkssj")).toInstant(ZoneOffset.of("+8")).toEpochMilli())).collect(Collectors.toList());

        List<Map<String,Map<String,Object>>> clcokInTimeData = Lists.newArrayList();

        for (Map<String, Object> needClockIn :needClockInSchedule){
            if (CheckBoxEnum.CHECKED.getKey().equals(needClockIn.get("ksdk"))){
                String dtkssj = analysisDate+" "+needClockIn.get("dtkssj");
                String dtjssj = analysisDate+" "+needClockIn.get("dtjssj");
                int tqdkfzs = Integer.valueOf(Util.null2String(needClockIn.get("tqdkfzs")));
                Map<String,Map<String,Object>> ksdkNearestClcokInTime = Utils.getNearestClcokInTimeCmd(dtkssj,clockInTimeList);
                String timeType = "empty";
                Map<String,Map<String,Object>> clcokInTimeMap = Maps.newHashMap();
                if (ksdkNearestClcokInTime.get("equal") != null){
                    timeType="equal";
                }
                if ("empty".equals(timeType) && ksdkNearestClcokInTime.get("before") != null){
                    String clockInTime = ksdkNearestClcokInTime.get("before").get("signdate") +" "+ksdkNearestClcokInTime.get("before").get("signtime");
                    if (DateUtil.getTime(clockInTime).compareTo(DateUtil.getTime(DateUtil.beforeMinutes(dtkssj,tqdkfzs))) >=0){
                        //打卡时间大于等于最早打卡时间
                        timeType="before";
                    }
                }
                if ("empty".equals(timeType) && ksdkNearestClcokInTime.get("after") != null){
                    String clockInTime = ksdkNearestClcokInTime.get("after").get("signdate") +" "+ksdkNearestClcokInTime.get("after").get("signtime");
                    if (DateUtil.getTime(clockInTime).compareTo(DateUtil.getTime(dtjssj)) < 0){
                        //打卡时间小于结束时间
                        timeType="after";
                    }
                }
                clcokInTimeMap.put(dtkssj+"|start|"+timeType,ksdkNearestClcokInTime.get(timeType));
                clcokInTimeData.add(clcokInTimeMap);
            }
            if (CheckBoxEnum.CHECKED.getKey().equals(needClockIn.get("jsdk"))){
                String dtkssj = analysisDate+" "+needClockIn.get("dtkssj");
                String dtjssj = analysisDate+" "+needClockIn.get("dtjssj");
                int thdkfzs = Integer.valueOf(Util.null2String(needClockIn.get("thdkfzs")));
                if (DateUtil.getTime(dtkssj).compareTo(DateUtil.getTime(dtjssj)) > 0){
                    dtjssj = DateUtil.AfterDay(analysisDate,1) +" "+needClockIn.get("dtjssj");
                }
                Map<String,Map<String,Object>> jsdkNearestClcokInTime = Utils.getNearestClcokInTimeCmd(dtjssj,clockInTimeList);

                String timeType = "empty";
                Map<String,Map<String,Object>> clcokInTimeMap = Maps.newHashMap();
                if (jsdkNearestClcokInTime.get("equal") != null){
                    timeType="equal";
                }
                if ("empty".equals(timeType) && jsdkNearestClcokInTime.get("after") != null){
                    String clockInTime = jsdkNearestClcokInTime.get("after").get("signdate") +" "+jsdkNearestClcokInTime.get("after").get("signtime");
                    if (DateUtil.getTime(clockInTime).compareTo(DateUtil.getTime(DateUtil.AfterMinutes(dtjssj,thdkfzs))) <=0){
                        timeType="after";
                    }
                }
                if ("empty".equals(timeType) && jsdkNearestClcokInTime.get("before") != null){
                    String clockInTime = jsdkNearestClcokInTime.get("before").get("signdate") +" "+jsdkNearestClcokInTime.get("before").get("signtime");
                    if (DateUtil.getTime(clockInTime).compareTo(DateUtil.getTime(dtkssj)) >0){
                        timeType="before";
                    }
                }
                clcokInTimeMap.put(dtjssj+"|end|"+timeType,jsdkNearestClcokInTime.get(timeType));
                clcokInTimeData.add(clcokInTimeMap);
            }
        }
        //当有2笔需要打卡时，可能会有打卡歧义的情况，歧义情况取2个时间点的中间值，当打卡时间小于中间值归属前一个打卡，大于则相反
        if (clcokInTimeData.size() >1){
            for (int i=0;i<clcokInTimeData.size()-1;i++){
                Map<String,Map<String,Object>> beforeClcokInTimeData = clcokInTimeData.get(i);
                Map<String,Map<String,Object>> afterClcokInTimeData = clcokInTimeData.get(i+1);
                Map<String,Object> beforeClcokInTimeMap = null;
                String beforeClcokInTime = "";
                String afterClcokInTime = "";
                Map<String,Object> afterClcokInTimeMap = null;
                for (Map.Entry<String,Map<String,Object>> beforeEntry :beforeClcokInTimeData.entrySet()){
                    beforeClcokInTimeMap = beforeEntry.getValue();
                    beforeClcokInTime = beforeEntry.getKey();
                }
                for (Map.Entry<String,Map<String,Object>> afterEntry :afterClcokInTimeData.entrySet()){
                    afterClcokInTimeMap = afterEntry.getValue();
                    afterClcokInTime = afterEntry.getKey();
                }
                //重复
                if (beforeClcokInTimeMap != null && beforeClcokInTimeMap == afterClcokInTimeMap ){
                    String beforeTime = beforeClcokInTime.split("\\|")[0];
                    String afterTime = afterClcokInTime.split("\\|")[0];
                    long betWeenMinutes = DateUtil.getBetWeenMinutes(beforeTime,afterTime);

                    String middileTime = DateUtil.AfterMinutes(beforeTime,betWeenMinutes/2);

                    String signdateTime = beforeClcokInTimeMap.get("signdate") +" "+beforeClcokInTimeMap.get("signtime");

                    if (DateUtil.getTime(signdateTime).compareTo(DateUtil.getTime(middileTime)) <=0){
                        //该打卡归属前一个打卡点
                        afterClcokInTimeData.put(afterClcokInTime,null);
                    }else if (DateUtil.getTime(signdateTime).compareTo(DateUtil.getTime(middileTime)) >0){
                        //该打卡归属后一个打卡点
                        beforeClcokInTimeData.put(beforeClcokInTime,null);
                    }
                }
            }
        }

    }


}
